@@ -91,7 +91,6 @@ from renpy.parser import unelide_filename, get_parse_errors
 from renpy.translation import change_language, known_languages
 from renpy.translation.generation import generic_filter as transform_text
 
-
 from renpy.persistent import register_persistent
 
 from renpy.character import show_display_say, predict_show_display_say, display_say
@@ -116,6 +115,8 @@ from renpy.loader import add_python_directory
 
 from renpy.lint import try_compile, try_eval
 
+from renpy.gl2.gl2shadercache import register_shader
+
 renpy_pure("ParameterizedText")
 renpy_pure("Keymap")
 renpy_pure("has_screen")
@@ -469,7 +470,7 @@ def can_show(name, layer=None, tag=None):
         return None
 
 
-def showing(name, layer='master'):
+def showing(name, layer=None):
     """
     :doc: image_func
 
@@ -481,7 +482,6 @@ def showing(name, layer='master'):
         component of the image name. It may also be a string giving
         only the image tag.
 
-
     `layer`
         The layer to check. If None, uses the default layer for `tag`.
     """
@@ -780,7 +780,7 @@ def scene(layer='master'):
         renpy.config.missing_scene(layer)
 
 
-def input(prompt, default='', allow=None, exclude='{}', length=None, with_none=None, pixel_width=None):  # @ReservedAssignment
+def input(prompt, default='', allow=None, exclude='{}', length=None, with_none=None, pixel_width=None, screen="input"):  # @ReservedAssignment
     """
     :doc: input
 
@@ -808,8 +808,18 @@ def input(prompt, default='', allow=None, exclude='{}', length=None, with_none=N
     `pixel_width`
         If not None, the input is limited to being this many pixels wide,
         in the font used by the input to display text.
+
+    `screen`
+        The name of the screen that takes input. If not given, the ``input``
+        screen is used.
+
+    If :var:`config.disable_input` is True, this function only returns
+    `default`.
     """
 
+    if renpy.config.disable_input:
+        return default
+
     renpy.exports.mode('input')
 
     roll_forward = renpy.exports.roll_forward_info()
@@ -822,14 +832,17 @@ def input(prompt, default='', allow=None, exclude='{}', length=None, with_none=N
 
     fixed = in_fixed_rollback()
 
-    if has_screen("input"):
+    if has_screen(screen):
         widget_properties = { }
         widget_properties["input"] = dict(default=default, length=length, allow=allow, exclude=exclude, editable=not fixed, pixel_width=pixel_width)
 
-        show_screen("input", _transient=True, _widget_properties=widget_properties, prompt=prompt)
+        show_screen(screen, _transient=True, _widget_properties=widget_properties, prompt=prompt)
 
     else:
 
+        if screen != "input":
+            raise Exception("The '{}' screen does not exist.".format(screen))
+
         renpy.ui.window(style='input_window')
         renpy.ui.vbox()
 
@@ -1075,10 +1088,21 @@ def display_menu(items,
     `screen`
         The name of the screen used to display the menu.
 
+    Note that most Ren'Py games do not use menu captions, but use narration
+    instead. To display a menu using narration, write::
+
+        $ narrator("Which direction would you like to go?", interact=False)
+        $ result = renpy.display_menu([ ("East", "east"), ("West", "west") ])
+
     """
 
+    menu_args, menu_kwargs = get_menu_args()
+    screen = menu_kwargs.pop("screen", screen)
+    with_none = menu_kwargs.pop("_with_none", with_none)
+    mode = menu_kwargs.pop("_mode", type)
+
     if interact:
-        renpy.exports.mode(type)
+        renpy.exports.mode(mode)
         choice_for_skipping()
 
     choices = [ ]
@@ -1113,9 +1137,6 @@ def display_menu(items,
 
     scope = dict(scope)
 
-    menu_args, menu_kwargs = get_menu_args()
-    screen = menu_kwargs.pop("screen", screen)
-
     scope.update(menu_kwargs)
 
     # Show the menu.
@@ -1174,6 +1195,8 @@ def display_menu(items,
                 **scope)
 
     else:
+        renpy.exports.shown_window()
+
         renpy.ui.window(style=window_style, focus="menu")
         renpy.ui.menu(items,
                       location=renpy.game.context().current,
@@ -1825,13 +1848,14 @@ def call(label, *args, **kwargs):
     raise renpy.game.CallException(label, args, kwargs, from_current=from_current)
 
 
-def return_statement():
+def return_statement(value=None):
     """
     :doc: se_call
 
     Causes Ren'Py to return from the current Ren'Py-level call.
     """
 
+    renpy.store._return = value
     jump("_renpy_return")
 
 
@@ -2655,7 +2679,7 @@ def get_placement(d):
     return placement(p)
 
 
-def get_image_bounds(tag, width=None, height=None, layer='master'):
+def get_image_bounds(tag, width=None, height=None, layer=None):
     """
     :doc: image_func
 
@@ -2853,14 +2877,20 @@ def call_screen(_screen_name, *args, **kwargs):
     to occur. The screen is hidden at the end of the interaction, and
     the result of the interaction is returned.
 
-    Keyword arguments not beginning with _ are passed to the scope of
-    the screen.
+    Positional arguments, and keyword arguments that do not begin with
+    _ are passed to the screen.
 
     If the keyword argument `_with_none` is false, "with None" is not
     run at the end of end of the interaction.
+
+    If the keyword argument `_mode` in kwargs, it will be mode of this
+    interaction, otherwise it will be "screen" mode.
     """
 
-    renpy.exports.mode('screen')
+    mode = "screen"
+    if "_mode" in kwargs:
+        mode = kwargs.pop("_mode")
+    renpy.exports.mode(mode)
 
     with_none = renpy.config.implicit_with_none
 
@@ -3009,6 +3039,19 @@ def notify(message):
 
     Only one notification is displayed at a time. If a second notification
     is displayed, the first notification is replaced.
+
+    This function just calls :var:`config.notify`, allowing its implementation
+    to be replaced by assigning a new function to that variable.
+    """
+
+    renpy.config.notify(message)
+
+
+def display_notify(message):
+    """
+    :doc: other
+
+    The default implementation of :func:`renpy.notify`.
     """
 
     hide_screen('notify')
@@ -3681,8 +3724,8 @@ def get_skipping():
     """
     :doc: other
 
-    Returns true if the Ren'Py is skipping, "fast" if Ren'Py is fast skipping,
-    and false if it is not skipping.
+    Returns "slow" if the Ren'Py is skipping, "fast" if Ren'Py is fast skipping,
+    and None if it is not skipping.
     """
 
     return renpy.config.skipping
@@ -3749,3 +3792,13 @@ def is_skipping():
     """
 
     return not not renpy.config.skipping
+
+
+def is_init_phase():
+    """
+    :doc: other
+
+    Returns True if Ren'Py is currently executing init code, or False otherwise.
+    """
+
+    return renpy.game.context().init_phase
